require 'commander/blank'
require 'commander/command'

module PolyglotCli
  module Command
    class Setup
      include Helper::Terminal
      include Helper::General

      def self.init(options)
        new(options).call
      end

      def initialize(options = Commander::Command::Options.new)
        @options = options
      end

      def call
        @config = {
          subdomain: subdomain_prompt,
          project_id: project_id_prompt,
          locale_path: locale_path_prompt
        }
        PolyglotCli::IO::Config.write(@config.merge(locale_mapping: locale_mapping))
        success
      rescue TTY::Prompt::ConfigurationError
        prompt.error('Could not find any projects. Please try a new search.')
      end

      # override subdomain for the setup,
      # use the cached value (from prompt)
      # instead of reading it from the config file
      # (which does not exist yet during the setup)
      attr_reader :subdomain

      private

      def locale_mapping
        return if mapping_prompt

        locale_map.each_with_object({}) do |map, hash|
          key = if map[1].size > 1
                  locale_prompt(map[0], map[1])
                else
                  map[1].first
                end
          hash[key] = map[0]
        end
      end

      def locale_map
        languages.each_with_object(Hash.new([])) do |l, hash|
          hash[l.locale[0..1]] += [l.locale]
        end
      end

      def mapping_prompt
        prompt.yes?('Would you like to include locales?')
      end

      def locale_prompt(language, locales)
        prompt.select("Choose a locale for #{language}: ", locales)
      end

      def subdomain_prompt
        @subdomain = prompt.ask('Enter your subdomain: ', default: 'infinum')
      end

      def project_id_prompt
        if filtered_projects.any?
          prompt.select('Choose a project: ', filtered_projects)
        else
          raise StandardError, "No projects for the subdomain #{subdomain}!"
        end
      end

      def locale_path_prompt
        prompt.ask('Where would you like to store your locale files?', default: 'config/locales')
      end

      def projects
        prompt.say('Getting projects...')
        PolyglotCli::Resource::Project.with_subdomain(subdomain).token(token).depaginate.each_with_object({}) do |r, hash|
          hash[r.name] = r.id
        end
      end

      def filtered_projects
        @filtered_projects ||= projects.select { |key, _id| key[/^(.*?(#{option_query})[^$]*)$/i] }.sort_by { |p| p[0].downcase }.to_h
      end

      def option_query
        @option_query ||= @options.__hash__.fetch(:query, '')
      end
    end
  end
end
